@@ -1,43 +1,7 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Copyright (C) 2020 Intel Corporation.
-** Copyright (C) 2019 Klarälvdalens Datakonsult AB.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// Copyright (C) 2020 Intel Corporation.
+// Copyright (C) 2019 Klarälvdalens Datakonsult AB.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qsharedpointer.h"
 
@@ -478,6 +442,46 @@
     get a compiler error.
 */
 
+/*!
+    \fn template <class T> QSharedPointer<T>::QSharedPointer(QSharedPointer &&other)
+
+    Move-constructs a QSharedPointer instance, making it point at the same
+    object that \a other was pointing to.
+
+    \since 5.4
+*/
+
+/*!
+    \fn template <class T> QSharedPointer<T>::operator=(QSharedPointer &&other)
+
+    Move-assigns \a other to this QSharedPointer instance.
+
+    \since 5.0
+*/
+
+/*!
+    \fn template <class T> template <class X> QSharedPointer<T>::QSharedPointer(QSharedPointer<X> &&other)
+
+    Move-constructs a QSharedPointer instance, making it point at the same
+    object that \a other was pointing to.
+
+    This constructor participates in overload resolution only if \c{X*}
+    implicitly converts to \c{T*}.
+
+    \since 5.6
+*/
+
+/*!
+    \fn template <class T> template <class X> QSharedPointer<T>::operator=(QSharedPointer<X> &&other)
+
+    Move-assigns \a other to this QSharedPointer instance.
+
+    This assignment operator participates in overload resolution only if \c{X*}
+    implicitly converts to \c{T*}.
+
+    \since 5.6
+*/
+
 /*!
     \fn template <class T> QSharedPointer<T>::QSharedPointer(const QWeakPointer<T> &other)
 
@@ -931,6 +935,15 @@
     Const overload of sharedFromThis().
 */
 
+/*!
+    \fn template <class T> qHash(const QSharedPointer<T> &key, size_t seed)
+    \relates QSharedPointer
+
+    Returns the hash value for \a key, using \a seed to seed the calculation.
+
+    \since 5.0
+*/
+
 /*!
     \fn template <class T> template <class X> bool operator==(const QSharedPointer<T> &ptr1, const QSharedPointer<X> &ptr2)
     \relates QSharedPointer
@@ -1450,7 +1463,7 @@ QT_END_NAMESPACE
 #  ifdef QT_SHARED_POINTER_BACKTRACE_SUPPORT
 #    if defined(__GLIBC__) && (__GLIBC__ >= 2) && !defined(__UCLIBC__) && !defined(QT_LINUXBASE)
 #      define BACKTRACE_SUPPORTED
-#    elif defined(Q_OS_MAC)
+#    elif defined(Q_OS_DARWIN)
 #      define BACKTRACE_SUPPORTED
 #    endif
 #  endif
