@@ -137,6 +137,9 @@ QT_BEGIN_NAMESPACE
 using namespace QImageReaderWriterHelpers;
 using namespace Qt::StringLiterals;
 
+Q_TRACE_POINT(qtgui, QImageReader_read_before_reading, QImageReader *reader, const QString &filename);
+Q_TRACE_POINT(qtgui, QImageReader_read_after_reading, QImageReader *reader, bool result);
+
 static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
                                                 const QByteArray &format,
                                                 bool autoDetectImageFormat,
@@ -460,7 +463,7 @@ public:
     static int maxAlloc;
 };
 
-int QImageReaderPrivate::maxAlloc = 128; // 128 MB is enough for an 8K 32bpp image
+int QImageReaderPrivate::maxAlloc = 256; // 256 MB is enough for an 8K 64bpp image
 
 /*!
     \internal
@@ -1569,6 +1572,8 @@ int QImageReader::allocationLimit()
     loading corrupt image files. It is normally not needed to change it. The
     default limit is large enough for all commonly used image sizes.
 
+    At runtime, this value may be overridden by the environment variable \c QT_IMAGEIO_MAXALLOC.
+
     \note The memory requirements are calculated for a minimum of 32 bits per pixel, since Qt will
     typically convert an image to that depth when it is used in GUI. This means that the effective
     allocation limit is significantly smaller than \a mbLimit when reading 1 bpp and 8 bpp images.
